#
# Licence GPL v3.
#
# Copyright Chris Johns (chrisj@rtems.org)
#
import optparse
import os
import pq
import pqconfig
import shutil
import sys
import tailer
import time

def publish_header(output, title):
    print >> output, '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">'
    print >> output, '<html>'
    print >> output, ' <head>'
    print >> output, '  <title>' + title + '</title>'
    print >> output, '  <meta http-equiv="Content-Language" content="English" />'
    print >> output, '  <meta http-equiv="Content-Type" content="text/html; charset=us-ascii" />'
    ss = pqconfig.stylesheet()
    if ss:
        print >> output, '  <link rel="stylesheet" type="text/css" href="' + ss + '" media="screen" />'
    print >> output, ' </head>'
    print >> output, ' <body>'

def publish_footer(output):
    print >> output, ' </body>'
    print >> output, '</html>'

def publish_heading(output, heading):
    print >> output, '  <div class="heading">'
    print >> output, '   ' + heading
    print >> output, '   <div class="datetime">' + time.ctime() + '</div>'
    print >> output, '  <small><br><a href="http://www.rtems.org/wiki/index.php/Build_Results">Help and details</a></small>'
    print >> output, '  </div>'

def publish_table_open(output, _class, cellspacing = 0):
    print >> output, '  <table class="' + _class + '" cellspacing="' + \
        str(cellspacing) + '" cellpadding="2">'

def publish_table_close(output):
    print >> output, '  </table>'

def publish_error_header(output):
    print >> output, '   <tr class="error-table-top"><th>Target</th>',
    print >> output, '<th>Dups</th><th>E</th><th>File</th><th>Error/Warning</th></tr>'

def publish_error(output, target, error, table_on):
    if table_on:
        print >> output, '    <tr class="error-table-on">',
    else:
        print >> output, '    <tr class="error-table-off">',
    print >> output, '<td>', pq.target_name(target, 3), '</td>',
    print >> output, '<td class="error-table-dups">', error['dup'], '</td>',
    if error['error'].type == 'error':
        print >> output, '<td class="error-table-error">&nbsp;</td>',
    else:
        print >> output, '<td class="error-table-warning">&nbsp;</td>',
    source_file = pqconfig.publish_source_file(error['error'].file, error['error'].line)
    if source_file:
        print >> output, '<td>', source_file, '</td>',
    else:
        print >> output, '<td>', '%s:%d' % (error['error'].file, error['error'].line), '</td>',
    print >> output, '<td>', error['error'].info, '</td>',
    print >> output, '</tr>'

def publish_navbar(output, first, prev, next, last):
    print >> output, '  <div class="navbar">'
    print >> output, '   <a href=".">Summary</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
    if first:
        print >> output, '   <a href="./' + first + '">First</a>'
    else:
        print >> output, '   First'
    print >> output, '   &nbsp;&nbsp;&nbsp;&nbsp;'
    if prev:
        print >> output, '   <a href="./' + prev + '">Prev</a>'
    else:
        print >> output, '   Prev'
    print >> output, '   &nbsp;&nbsp;&nbsp;&nbsp;'
    if next:
        print >> output, '   <a href="./' + next + '">Next</a>'
    else:
        print >> output, '   Next'
    print >> output, '   &nbsp;&nbsp;&nbsp;&nbsp;'
    if last:
        print >> output, '   <a href="./' + last + '">Last</a>'
    else:
        print >> output, '   Last'
    print >> output, '  </div>'
    
def publish_target_stats(output, outpath, workspace, target, history):
    publish_table_open(output, 'stats-table', 1)
    print >> output, '   <tr class="stats-table-target"><td colspan="8">',
    print >> output, pq.target_name(target, 3), '</td></tr>'
    print >> output, '   <tr class="stats-table-top"><th>Date</th>',
    print >> output, '<th>Errors</th><th>Warnings</th><th>Notes</th>',
    print >> output, '<th>Duplicates</th><th>Status</th><th>Results</th><th>Output</th></tr>'
    for h in history.history:
        if h[0] == target:
            t = history.date2time(h[1])
            filename_base = pqconfig.publish_filename(h[0], t)
            src_tarball = pqconfig.src_tarball(history.date2time(h[1]))
            print >> output, '   <tr class="stats-table-row">',
            print >> output, '<td align="center">',
            if os.path.isfile(os.path.join(outpath, src_tarball)):
                print >> output, '<a href="' + src_tarball + '">', h[1], '</a>',
            else:
                print >> output, h[1],
            print >> output, '</td>',
            print >> output, '<td>', h[2], '</td>',
            print >> output, '<td>', h[3], '</td>',
            print >> output, '<td>', h[4], '</td>',
            print >> output, '<td>', h[5], '</td>',
            if h[6] == 'PASSED':
                _class = 'stats-target-good'
            else:
                _class = 'stats-target-bad'
            print >> output, '<td class="' + _class + '">', h[6], '</td>',
            print >> output, '<td class="stats-target-results">'
            name = filename_base + '-1.html'
            if os.path.isfile(os.path.join(outpath, name)):
                print >> output, '<a href="' + name + '">Results</a>',
            else:
                print >> output, ' ',
            print >> output, '</td><td align="center">',
            name = filename_base + '-tail.html'
            if os.path.isfile(os.path.join(outpath, name)):
                print >> output, '<a href="' + name + '">Output</a>',
            else:
                print >> output, ' ',
            config = filename_base + '-config.log'
            archive = filename_base + '-output' + pqconfig.build_output_suffix()
            if os.path.isfile(os.path.join(outpath, config)) or \
                    os.path.isfile(os.path.join(outpath, archive)):
                print >> output, '&nbsp;<small><i>(',
                if os.path.isfile(os.path.join(outpath, config)):
                    print >> output, '<a href="' + config + '">config</a>',
                    if os.path.isfile(os.path.join(outpath, archive)):
                        print >> output, ',',
                if os.path.isfile(os.path.join(outpath, archive)):
                    print >> output, '<a href="' + archive + '">full</a>',
                print >> output, ')</i></small>',
            print >> output, '</td>',
            print >> output, '</tr>'
    publish_table_close(output)

def publish_output_tail(output, target, tail):
    publish_heading(output, pqconfig.build_output_title(target))
    print >> output, '<pre class="build-output">'
    for l in tail:
        print >> output, l
    print >> output, '</pre>'

def publish_output_config_log(output, workspace, target, filename_base):
    cfg = os.path.join(workspace, target, pqconfig.build_output_config_log())
    if not os.path.isfile(cfg):
        print >> sys.stderr, "cannot find the config log:", cfg
    else:
        if not os.path.exists(output) or not os.path.isdir(output):
            os.makedirs(output)
        try:
            dst = os.path.join(output, 
                               filename_base + '-config.log')
            shutil.copy(cfg, dst)
        except IOError, e:
            print >> sys.stderr, "Cannot copy config log:", e
            pass

def publish_output_archive(output, workspace, target, filename_base):
    out = os.path.join(workspace, target, pqconfig.build_output_archive())
    if not os.path.isfile(out):
        print >> sys.stderr, "cannot find the output archive:", out
    else:
        if not os.path.exists(output) or not os.path.isdir(output):
            os.makedirs(output)
        try:
            dst = os.path.join(output, 
                               filename_base + '-output' + pqconfig.build_output_suffix())
            shutil.copy(out, dst)
        except IOError, e:
            print >> sys.stderr, "Cannot copy output archive:", e
            pass

def publish_source_tarball(output, workspace, t):
    src = os.path.join(workspace, pqconfig.src_tarball(t))
    if not os.path.isfile(src):
        print >> sys.stderr, "cannot find the source tarball:", src
    else:
        if not os.path.exists(output) or not os.path.isdir(output):
            os.makedirs(output)
        try:
            shutil.copy(src, output)
        except IOError, e:
            print >> sys.stderr, "Cannot copy source tarball:", e
            pass

def queue_publish(base, data, workspace, source, targets, output, date):
    t = time.mktime(time.strptime(date, '%Y%m%d'))
    history = pq.history(data, output)
    pqconfig.publish_config_setup(base, output)
    index = open(os.path.join(output, 'index.html'), 'w') 
    publish_header(index, pqconfig.title())
    publish_source_tarball (output, workspace, t)
    publish_heading(index, pqconfig.title())
    for target in targets:
        try:
            errors = pq.errors(base, data, source, pqconfig.source_find)
            stdout = open(os.path.join(workspace, target, 'stdout.txt'))
            errors.load_errs(target, stdout)
            tail = tailer.tail(stdout, pqconfig.build_output_tail())
            stdout.close()
        except IOError, e:
            print >> sys.stderr, "cannot open the target stdout:", target
            continue
        ofile = None
        count = 0
        new_page = True
        page = 0
        pages = (errors.count() / pqconfig.errors_per_page())
        filename_base = pqconfig.publish_filename(target, t)
        first_filename = filename_base + '-1.html'
        last_filename = filename_base + '-%d.html' % (pages)
        prev_filename = None
        next_filename = None
        filename = None
        for error in errors.walk_errs():
            if errors.check_all(target, error['error'].error):
                continue
            if new_page:
                count = 0
                page += 1
                new_page = False
                if ofile:
                    publish_table_close(ofile)
                    publish_navbar(ofile, first_filename, 
                                   prev_filename, next_filename,
                                   last_filename)
                    publish_footer(ofile)
                    ofile.close()
                prev_filename = filename
                filename = filename_base + '-%d.html' % (page)
                if page < pages:
                    next_filename = filename_base + '-%d.html' % (page + 1)
                else:
                    next_filename = None
                ofile = open(os.path.join(output, filename), 'w')
                publish_header(ofile, pqconfig.target_title(target, page))
                publish_heading(ofile, pqconfig.target_title(target, page))
                publish_table_open(ofile, 'error-table')
                publish_error_header(ofile)
                table_on = False
            publish_error(ofile, target, error, table_on)
            table_on = not table_on
            count += 1
            if count >= pqconfig.errors_per_page():
                new_page = True
        if ofile:
            publish_table_close(ofile)
            publish_navbar(ofile, first_filename, 
                           prev_filename, next_filename,
                           last_filename)
            publish_footer(ofile)
            ofile.close()
        publish_output_config_log(output, workspace, target, filename_base)
        publish_output_archive(output, workspace, target, filename_base)
        filename = filename_base + '-tail.html'
        ofile = open(os.path.join(output, filename), 'w')
        publish_header(ofile, pqconfig.target_title(target, page))
        publish_output_tail(ofile, target, tail)
        publish_footer(ofile)
        ofile.close()
        history.add(workspace, target, errors, t)
        history.remove_oldest(target)
        publish_target_stats(index, output, workspace, target, history)
        del errors
    publish_footer(index)
    index.close()
    history.unload(data)

#
# Command line options.
#
cl_options = [
    optparse.make_option('-o', action = 'store', type = 'string', 
                         dest = 'output', default = None,
                         help = 'Directory to write the HTML output to'),
    optparse.make_option('-D', action = 'store', type = 'string', 
                         dest = 'date', default = None,
                         help = 'Date of the source tarball'),
    # Standard options
    optparse.make_option('-d', action = 'store', type = 'string', 
                         dest = 'data', default = os.environ['HOME'],
                         help = 'Specify the data directory'),
    optparse.make_option('-w', action = 'store', type = 'string', 
                         dest = 'workspace', default = None,
                         help = 'Specify the workspace directory'),
    optparse.make_option('-s', action = 'store', type = 'string', 
                         dest = 'source', default = None,
                         help = 'Specify the source directory'),
    optparse.make_option('-t', action = 'store', type = 'string', 
                         dest = 'targets', default = None,
                         help = 'List of comma seperated targets'),
    optparse.make_option('-c', action = 'store', type = 'string',
                         dest = 'config',
                         default = os.environ['HOME'] + '/.pq-config',
                         help = 'Configration file name to read')
]

def user_main(base, argv):
    (options, config, data, workspace, source, targets) = \
        pq.parse_std_args(argv, cl_options)
    if not options.output:
        print >> sys.stderr, 'no output directory specified'
        sys.exit(1)
    if not options.date:
        print >> sys.stderr, 'no source tarball date specified'
        sys.exit(1)
    queue_publish(base, data, workspace, source, targets, options.output, options.date)
